@@ -9,11 +9,13 @@
 #
 import operator
 from decimal import Decimal
-from typing import Any, Tuple, Union
-from .atomic_types import AtomicTypeMeta, AnyAtomicType
+from typing import Any, Optional, Tuple, Union
+
+from ..helpers import BOOLEAN_VALUES, get_double
+from .atomic_types import AnyAtomicType
 
 
-class UntypedAtomic(metaclass=AtomicTypeMeta):
+class UntypedAtomic(AnyAtomicType):
     """
     Class for xs:untypedAtomic data. Provides special methods for comparing
     and converting to basic data types.
@@ -61,22 +63,24 @@ class UntypedAtomic(metaclass=AtomicTypeMeta):
         """
         if isinstance(other, UntypedAtomic):
             if force_float:
-                return float(self.value), float(other.value)
+                return get_double(self.value), get_double(other.value)
             return self.value, other.value
         elif isinstance(other, bool):
             # Cast to xs:boolean
             value = self.value.strip()
-            if value not in {'0', '1', 'true', 'false'}:
+            if value not in BOOLEAN_VALUES:
                 raise ValueError("{!r} cannot be cast to xs:boolean".format(self.value))
             return value in ('1', 'true'), other
         elif isinstance(other, int):
-            return float(self.value), other
-        elif isinstance(other, str):
-            return str(self.value), other
+            return get_double(self.value), other
+        elif other is None or isinstance(other, (str, list)):
+            return self.value, other
 
-        try:
+        if hasattr(other, 'fromstring'):
             return type(other).fromstring(self.value), other
-        except AttributeError:
+        elif hasattr(other, 'ordered'):
+            return type(other)(self.value, other.ordered), other
+        else:
             return type(other)(self.value), other
 
     def __hash__(self) -> int:
@@ -124,7 +128,7 @@ class UntypedAtomic(metaclass=AtomicTypeMeta):
         return int(self.value)
 
     def __float__(self) -> float:
-        return float(self.value)
+        return get_double(self.value, xsd_version='1.1')
 
     def __bool__(self) -> bool:
         return bool(self.value)  # For effective boolean value, not for cast to xs:boolean.
@@ -135,6 +139,9 @@ class UntypedAtomic(metaclass=AtomicTypeMeta):
     def __mod__(self, other: Any) -> Any:
         return operator.mod(*self._get_operands(other))
 
+    def __round__(self, n: Optional[int] = None) -> float:
+        return round(float(self.value), ndigits=n)
+
     def __str__(self) -> str:
         return self.value
 
