@@ -31,8 +31,7 @@ from .xpath_nodes import XPathNode, ElementNode, AttributeNode, \
     DocumentNode, NamespaceNode, SchemaElementNode
 from .datatypes import xsd10_atomic_types, AbstractDateTime, AnyURI, \
     UntypedAtomic, Timezone, DateTime10, Date10, DayTimeDuration, Duration, \
-    Integer, DoubleProxy10, DoubleProxy, QName, DatetimeValueType, \
-    AtomicValueType, AnyAtomicType
+    Integer, DoubleProxy10, DoubleProxy, QName, AtomicValueType, AnyAtomicType
 from .protocols import ElementProtocol, DocumentProtocol, XsdAttributeProtocol, \
     XsdElementProtocol, XsdTypeProtocol, XsdSchemaProtocol
 from .sequence_types import is_sequence_type_restriction, match_sequence_type
@@ -49,6 +48,8 @@ if TYPE_CHECKING:
 else:
     XPathParserType = Any
 
+ContextArgType = Optional[XPathContext]
+
 _XSD_SPECIAL_TYPES = {XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE}
 
 _CHILD_AXIS_TOKENS = {
@@ -91,7 +92,7 @@ class XPathToken(Token[XPathTokenType]):
     namespace = None  # for namespace binding of names and wildcards
     occurrence = None  # occurrence indicator for item types
 
-    def evaluate(self, context: Optional[XPathContext] = None) -> Any:
+    def evaluate(self, context: ContextArgType = None) -> Any:
         """
         Evaluate default method for XPath tokens.
 
@@ -99,7 +100,7 @@ class XPathToken(Token[XPathTokenType]):
         """
         return [x for x in self.select(context)]
 
-    def select(self, context: Optional[XPathContext] = None) -> Iterator[Any]:
+    def select(self, context: ContextArgType = None) -> Iterator[Any]:
         """
         Select operator that generates XPath results.
 
@@ -253,7 +254,7 @@ class XPathToken(Token[XPathTokenType]):
 
     ###
     # Dynamic context methods
-    def get_argument(self, context: Optional[XPathContext],
+    def get_argument(self, context: ContextArgType,
                      index: int = 0,
                      required: bool = False,
                      default_to_context: bool = False,
@@ -275,7 +276,8 @@ class XPathToken(Token[XPathTokenType]):
         :param cls: if a type is provided performs a type checking on item.
         :param promote: a class or a tuple of classes that are promoted to `cls` class.
         """
-        item: Union[None, ElementProtocol, DocumentProtocol, XPathNode, AnyAtomicType]
+        item: Union[None, ElementProtocol, DocumentProtocol,
+                    XPathNode, AnyAtomicType, XPathFunction]
 
         try:
             token = self._items[index]
@@ -284,6 +286,8 @@ class XPathToken(Token[XPathTokenType]):
                 if context is None:
                     raise self.missing_context() from None
                 item = context.item if context.item is not None else context.root
+            elif isinstance(context, XPathSchemaContext):
+                return default
             elif required:
                 msg = "missing %s argument" % ordinal(index + 1)
                 raise self.error('XPST0017', msg) from None
@@ -309,7 +313,7 @@ class XPathToken(Token[XPathTokenType]):
                     raise self.error('XPTY0004', msg)
             else:
                 if item is None:
-                    if not required:
+                    if not required or isinstance(context, XPathSchemaContext):
                         return default
                     ord_arg = ordinal(index + 1)
                     msg = "A not empty sequence required for {} argument"
@@ -333,7 +337,7 @@ class XPathToken(Token[XPathTokenType]):
                 tokens.append(tk)
                 return tokens[::-1]
 
-    def get_function(self, context: Optional[XPathContext], arity: int = 0) -> 'XPathFunction':
+    def get_function(self, context: ContextArgType, arity: int = 0) -> 'XPathFunction':
         if isinstance(self, XPathFunction):
             func = self
         elif self.symbol in (':', 'Q{') and isinstance(self[1], XPathFunction):
@@ -393,7 +397,7 @@ class XPathToken(Token[XPathTokenType]):
             msg = f"{ordinal(index+1)} argument has type {type(item)!r} instead of {cls!r}"
         raise self.error(code, msg)
 
-    def iter_flatten(self, context: Optional[XPathContext] = None) -> Iterator[Any]:
+    def iter_flatten(self, context: ContextArgType = None) -> Iterator[Any]:
 
         def _iter_flatten(items: Iterable[Any]) -> Iterator[Any]:
             for item in items:
@@ -406,7 +410,7 @@ class XPathToken(Token[XPathTokenType]):
 
         yield from _iter_flatten(self.select(context))
 
-    def atomization(self, context: Optional[XPathContext] = None) \
+    def atomization(self, context: ContextArgType = None) \
             -> Iterator[AtomicValueType]:
         """
         Helper method for value atomization of a sequence.
@@ -438,7 +442,7 @@ class XPathToken(Token[XPathTokenType]):
                 msg = f"sequence item {item!r} is not appropriate for the context"
                 raise self.error('XPTY0004', msg)
 
-    def get_atomized_operand(self, context: Optional[XPathContext] = None) \
+    def get_atomized_operand(self, context: ContextArgType = None) \
             -> Optional[AtomicValueType]:
         """
         Get the atomized value for an XPath operator.
@@ -480,7 +484,7 @@ class XPathToken(Token[XPathTokenType]):
                 msg = "atomized operand is a sequence of length greater than one"
                 raise self.error('XPTY0004', msg)
 
-    def iter_comparison_data(self, context: XPathContext) -> Iterator[OperandsType]:
+    def iter_comparison_data(self, context: ContextArgType) -> Iterator[OperandsType]:
         """
         Generates comparison data couples for the general comparison of sequences.
         Different sequences maybe generated with an XPath 2.0 parser, depending on
@@ -539,32 +543,36 @@ class XPathToken(Token[XPathTokenType]):
 
             yield values
 
-    def select_results(self, context: Optional[XPathContext]) -> Iterator[XPathResultType]:
+    def select_results(self, context: ContextArgType) -> Iterator[XPathResultType]:
         """
         Generates formatted XPath results.
 
         :param context: the XPath dynamic context.
         """
-        if context is not None:
+        if context is None:
+            yield from self.select(context)
+        else:
             self.parser.check_variables(context.variables)
 
-        for result in self.select(context):
-            if not isinstance(result, XPathNode):
-                yield result
-            elif isinstance(result, NamespaceNode):
-                if self.parser.compatibility_mode:
-                    yield result.prefix, result.uri
-                else:
-                    yield result.uri
-            elif isinstance(result, DocumentNode):
-                if result.is_extended():
+            for result in self.select(context):
+                if not isinstance(result, XPathNode):
                     yield result
+                elif isinstance(result, NamespaceNode):
+                    if self.parser.compatibility_mode:
+                        yield result.prefix, result.uri
+                    else:
+                        yield result.uri
+                elif isinstance(result, DocumentNode):
+                    if result.is_extended():
+                        # cannot represent with an ElementTree: yield the document node
+                        yield result
+                    elif result is context.root or result is not context.document:
+                        yield result.value
                 else:
                     yield result.value
-            else:
-                yield result.value
 
-    def get_results(self, context: XPathContext) -> Union[List[XPathResultType], AtomicValueType]:
+    def get_results(self, context: ContextArgType) \
+            -> Union[List[XPathResultType], AtomicValueType]:
         """
         Returns results formatted according to XPath specifications.
 
@@ -572,26 +580,28 @@ class XPathToken(Token[XPathTokenType]):
         :return: a list or a simple datatype when the result is a single simple type \
         generated by a literal or function token.
         """
-        if context is not None:
+        item = None
+        if context is None:
+            results = [x for x in self.select(context)]
+        else:
             self.parser.check_variables(context.variables)
 
-        results = []
-        item = None
-        for item in self.select(context):
-            if not isinstance(item, XPathNode):
-                results.append(item)
-            elif isinstance(item, NamespaceNode):
-                if self.parser.compatibility_mode:
-                    results.append((item.prefix, item.uri))
-                else:
-                    results.append(item.uri)
-            elif isinstance(item, DocumentNode):
-                if item.is_extended():
+            results = []
+            for item in self.select(context):
+                if not isinstance(item, XPathNode):
                     results.append(item)
+                elif isinstance(item, NamespaceNode):
+                    if self.parser.compatibility_mode:
+                        results.append((item.prefix, item.uri))
+                    else:
+                        results.append(item.uri)
+                elif isinstance(item, DocumentNode):
+                    if item.is_extended():
+                        results.append(item)
+                    elif item is not context.document or item is context.root:
+                        results.append(item.value)
                 else:
                     results.append(item.value)
-            else:
-                results.append(item.value)
 
         if len(results) == 1 and not isinstance(item, (ElementNode, DocumentNode)):
             if isinstance(item, (bool, int, float, Decimal)):
@@ -601,7 +611,7 @@ class XPathToken(Token[XPathTokenType]):
 
         return results
 
-    def get_operands(self, context: XPathContext, cls: Optional[Type[Any]] = None) \
+    def get_operands(self, context: ContextArgType, cls: Optional[Type[Any]] = None) \
             -> OperandsType:
         """
         Returns the operands for a binary operator. Float arguments are converted
@@ -659,14 +669,14 @@ class XPathToken(Token[XPathTokenType]):
         """
         Obtains an absolute URI from the argument and the static context.
 
-        :param uri: a string representing an URI.
+        :param uri: a string representing a URI.
         :param base_uri: an alternative base URI, otherwise the base_uri \
         of the static context is used.
         :param as_string: if `True` then returns the URI as a string, otherwise \
         returns the URI as xs:anyURI instance.
-        :returns: the argument if it's an absolute URI. Otherwise returns the URI
+        :returns: the argument if it's an absolute URI, otherwise returns the URI
         obtained by the join o the base_uri of the static context with the
-        argument. Returns the argument if the base_uri is `None'.
+        argument. Returns the argument if the base_uri is `None`.
         """
         if not base_uri:
             base_uri = self.parser.base_uri
@@ -733,8 +743,8 @@ class XPathToken(Token[XPathTokenType]):
 
         self.namespace = namespace
 
-    def adjust_datetime(self, context: XPathContext, cls: Type[DatetimeValueType]) \
-            -> Union[List[Any], DatetimeValueType, DayTimeDuration]:
+    def adjust_datetime(self, context: ContextArgType, cls: Type[AbstractDateTime]) \
+            -> Union[List[Any], AbstractDateTime, DayTimeDuration]:
         """
         XSD datetime adjust function helper.
 
@@ -744,8 +754,8 @@ class XPathToken(Token[XPathTokenType]):
         or the adjusted XSD datetime instance.
         """
         timezone: Optional[Any]
-        item: Optional[DatetimeValueType]
-        _item: Union[DatetimeValueType, DayTimeDuration]
+        item: Optional[AbstractDateTime]
+        _item: Union[AbstractDateTime, DayTimeDuration]
 
         if len(self) == 1:
             item = self.get_argument(context, cls=cls)
@@ -760,7 +770,10 @@ class XPathToken(Token[XPathTokenType]):
                 try:
                     timezone = Timezone.fromduration(timezone)
                 except ValueError as err:
-                    raise self.error('FODT0003', str(err)) from None
+                    if isinstance(context, XPathSchemaContext):
+                        timezone = Timezone.fromduration(DayTimeDuration(0))
+                    else:
+                        raise self.error('FODT0003', str(err)) from None
             if item is None:
                 return []
 
@@ -776,6 +789,8 @@ class XPathToken(Token[XPathTokenType]):
                     _item -= timezone.offset - _tzinfo.offset
                     _item -= DayTimeDuration.fromstring('P1D')
         except OverflowError as err:
+            if isinstance(context, XPathSchemaContext):
+                return _item
             raise self.error('FODT0001', str(err)) from None
 
         if not isinstance(_item, DayTimeDuration):
@@ -1158,10 +1173,10 @@ class ValueToken(XPathToken):
     def source(self) -> str:
         return str(self.value)
 
-    def evaluate(self, context: Optional[XPathContext] = None) -> Any:
+    def evaluate(self, context: ContextArgType = None) -> Any:
         return self.value
 
-    def select(self, context: Optional[XPathContext] = None) -> Iterator[Any]:
+    def select(self, context: ContextArgType = None) -> Iterator[Any]:
         if isinstance(self.value, list):
             yield from self.value
         elif self.value is not None:
@@ -1205,13 +1220,13 @@ class XPathFunction(XPathToken):
     pattern = r'(?<!\$)\b[^\d\W][\w.\-\xb7\u0300-\u036F\u203F\u2040]*' \
               r'(?=\s*(?:\(\:.*\:\))?\s*\((?!\:))'
 
-    sequence_types: Tuple[str, ...] = ()
+    sequence_types: Union[List[str], Tuple[str, ...]] = ()
     "Sequence types of arguments and of the return value of the function."
 
     nargs: NargsType = None
     "Number of arguments: a single value or a couple with None that means unbounded."
 
-    context: Optional[XPathContext] = None
+    context: ContextArgType = None
     "Dynamic context associated by function reference evaluation."
 
     def __init__(self, parser: 'XPath1Parser', nargs: Optional[int] = None) -> None:
@@ -1245,7 +1260,7 @@ class XPathFunction(XPathToken):
             return f"'Q{{{namespace}}}{self.symbol}' {self.label}"
 
     def __call__(self, *args: XPathFunctionArgType,
-                 context: Optional[XPathContext] = None) -> Any:
+                 context: ContextArgType = None) -> Any:
         self.check_arguments_number(len(args))
 
         # Check provided argument with arity
@@ -1494,10 +1509,10 @@ class XPathFunction(XPathToken):
         assert nargs, "a partial function requires at least a placeholder token"
 
         if self.label != 'partial function':
-            def evaluate(context: Optional[XPathContext] = None) -> Any:
+            def evaluate(context: ContextArgType = None) -> Any:
                 return self
 
-            def select(context: Optional[XPathContext] = None) -> Any:
+            def select(context: ContextArgType = None) -> Any:
                 yield self
 
             if self.__class__.evaluate is not XPathToken.evaluate:
@@ -1512,10 +1527,10 @@ class XPathFunction(XPathToken):
         self.label = 'partial function'
         self.nargs = nargs
 
-    def _partial_evaluate(self, context: Optional[XPathContext] = None) -> Any:
+    def _partial_evaluate(self, context: ContextArgType = None) -> Any:
         return [x for x in self._partial_select(context)]
 
-    def _partial_select(self, context: Optional[XPathContext] = None) -> Iterator[Any]:
+    def _partial_select(self, context: ContextArgType = None) -> Iterator[Any]:
         item = self._partial_evaluate(context)
         if item is not None:
             if isinstance(item, list):
@@ -1619,7 +1634,7 @@ class XPathMap(XPathFunction):
             items = ', '.join(f'{k!r}:{v!r}' for k, v in self._map.items())
         return f'map{{{items}}}'
 
-    def evaluate(self, context: Optional[XPathContext] = None) -> 'XPathMap':
+    def evaluate(self, context: ContextArgType = None) -> 'XPathMap':
         if self._map is not None:
             return self
         return XPathMap(
@@ -1630,7 +1645,7 @@ class XPathMap(XPathFunction):
             )
         )
 
-    def _evaluate(self, context: Optional[XPathContext] = None) -> Dict[AnyAtomicType, Any]:
+    def _evaluate(self, context: ContextArgType = None) -> Dict[AnyAtomicType, Any]:
         _map: Dict[Any, Any] = {}
         nan_key = None
 
@@ -1656,14 +1671,16 @@ class XPathMap(XPathFunction):
         return cast(Dict[AnyAtomicType, Any], _map)
 
     def __call__(self, *args: XPathFunctionArgType,
-                 context: Optional[XPathContext] = None) -> Any:
+                 context: ContextArgType = None) -> Any:
         if len(args) == 1 and isinstance(args[0], list) and len(args[0]) == 1:
             args = args[0][0],
         if len(args) != 1 or not isinstance(args[0], AnyAtomicType):
+            if isinstance(context, XPathSchemaContext):
+                return None
             raise self.error('XPST0003', 'exactly one atomic argument is expected')
 
         map_dict: Dict[Any, Any]
-        key = cast(AnyAtomicType, args[0])
+        key = args[0]
         if self._map is not None:
             map_dict = self._map
         else:
@@ -1677,19 +1694,19 @@ class XPathMap(XPathFunction):
         except KeyError:
             return []
 
-    def keys(self, context: Optional[XPathContext] = None) -> List[AnyAtomicType]:
+    def keys(self, context: ContextArgType = None) -> List[AnyAtomicType]:
         if self._map is not None:
             keys = [self._nan_key if k is None else k for k in self._map.keys()]
         else:
             keys = [self._nan_key if k is None else k for k in self._evaluate(context).keys()]
         return cast(List[AnyAtomicType], keys)
 
-    def values(self, context: Optional[XPathContext] = None) -> List[Any]:
+    def values(self, context: ContextArgType = None) -> List[Any]:
         if self._map is not None:
             return [v for v in self._map.values()]
         return [v for v in self._evaluate(context).values()]
 
-    def items(self, context: Optional[XPathContext] = None) -> List[Tuple[AnyAtomicType, Any]]:
+    def items(self, context: ContextArgType = None) -> List[Tuple[AnyAtomicType, Any]]:
         _map: Dict[Any, Any]
         if self._map is not None:
             _map = self._map
@@ -1782,12 +1799,12 @@ class XPathArray(XPathFunction):
         self.parser.advance('}')
         return self
 
-    def evaluate(self, context: Optional[XPathContext] = None) -> 'XPathArray':
+    def evaluate(self, context: ContextArgType = None) -> 'XPathArray':
         if self._array is not None:
             return self
         return XPathArray(self.parser, items=self._evaluate(context))
 
-    def _evaluate(self, context: Optional[XPathContext] = None) -> List[Any]:
+    def _evaluate(self, context: ContextArgType = None) -> List[Any]:
         if self.symbol == 'array':
             # A comma in a curly array constructor is the comma operator, not a delimiter.
             items: List[Any] = []
@@ -1798,7 +1815,7 @@ class XPathArray(XPathFunction):
             return [tk.evaluate(context) for tk in self._items]
 
     def __call__(self, *args: XPathFunctionArgType,
-                 context: Optional[XPathContext] = None) -> Any:
+                 context: ContextArgType = None) -> Any:
         if len(args) != 1 or not isinstance(args[0], int):
             raise self.error('XPTY0004', 'exactly one xs:integer argument is expected')
 
@@ -1816,12 +1833,12 @@ class XPathArray(XPathFunction):
         except IndexError:
             raise self.error('FOAY0001')
 
-    def items(self, context: Optional[XPathContext] = None) -> List[Any]:
+    def items(self, context: ContextArgType = None) -> List[Any]:
         if self._array is not None:
             return self._array.copy()
         return self._evaluate(context)
 
-    def iter_flatten(self, context: Optional[XPathContext] = None) -> Iterator[Any]:
+    def iter_flatten(self, context: ContextArgType = None) -> Iterator[Any]:
         if self._array is not None:
             items = self._array
         else:
